from enum import Enum

from wedeliver_core.helpers.service_config import ServiceConfig


class Service:
    CAPTAIN = ServiceConfig('CAPTAIN_SERVICE')
    FINANCE = ServiceConfig('FINANCE_SERVICE')
    SDD = ServiceConfig('SDD_SERVICE')
    SUPPLIER = ServiceConfig('SUPPLIER_SERVICE')
    PN = ServiceConfig('PN_SERVICE')
    FINTECH = ServiceConfig('FINTECH_SERVICE')
    STC = ServiceConfig('STC_SERVICE')
    AUTH = ServiceConfig('AUTH_SERVICE')
    MAIL = ServiceConfig('MAIL_SERVICE')
    SMS = ServiceConfig('SMS_SERVICE')
    APILAYER = ServiceConfig('APILAYER_SERVICE')
    INVOICE = ServiceConfig('INVOICE_SERVICE')
    ADDRESS = ServiceConfig('ADDRESS_SERVICE')
    PUBLIC = ServiceConfig('PUBLIC_SERVICE')
    INTERNAL_NOTIFICATION = ServiceConfig('INTERNAL_NOTIFICATION_SERVICE')


class QueryTypes(Enum):
    SIMPLE_TABLE = 1
    FUNCTION = 2
    SEARCH = 3


class InstallmentType(Enum):
    LEASE = 1
    PERSONAL_LOAN = 2


def list_enum_values(enum_type):
    "return list of dict the enum values and names for the given enum type"
    return [dict(id=e.value, Value=e.name.capitalize()) for e in enum_type]


def get_enum_value(enum_value, enum_type):
    "return  dict the enum values and names for the given enum type"
    for e in enum_type:
        if e.value == enum_value:
            return dict(id=e.value, Value=e.name.replace("_", " ").capitalize())


def format_values_dict(**values):
    "return  dict the enum values and names for the given enum type"
    return dict(**values)


class OrderByEnum(Enum):
    asc = "asc"
    desc = "desc"


def format_engine_size(engine_size_enum_type):
    "return the engine size in the format of 1.0 L"
    return [
        dict(id=e.value, Value=e.name[1:].replace("_", ".").capitalize() + " L")
        for e in engine_size_enum_type
    ]


def format_enum_with_dash(engine_size_enum_type):
    "return the enum containing dash to be without dash"
    return [
        dict(id=e.value, Value=e.name.replace("_", " ").capitalize())
        for e in engine_size_enum_type
    ]



from wedeliver_core.helpers.sql import sql


def search_function(table_name, search_list, append_extra=None):
    """
    """
    validation_error = []

    def _find_matched(db_results, key, search_item, operator=None):
        for db_instance in db_results:
            if operator == 'LIKE':
                if search_item.get("search_value") in db_instance.get(key):
                    return db_instance
            else:
                if search_item.get("search_value") == db_instance.get(key):
                    return db_instance
        return None

    for item_dict in search_list:
        # query = db.session.query(model)

        input_group_search_key = item_dict.get('search_key') or "id"

        input_group = item_dict.get('inputs')
        input_group_operator = item_dict.get('operator')
        input_group_list = [obj.get("search_value") for obj in input_group]

        # model_key = model.__dict__.get(input_group_search_key)
        # result_db = query.filter(model_key.in_(input_group_list)).all()
        # result_db_list = [obj.__dict__.get(input_group_search_key) for obj in result_db]

        if input_group_operator == "LIKE":
            where_condition = "RLIKE '{input_group_list}'".format(
                input_group_list='|'.join(str(value) for value in input_group_list),
            )
        else:
            where_condition = "IN ('{input_group_list}')".format(
                input_group_list='\',\''.join(str(value) for value in input_group_list),
            )
        query = """SELECT *
                FROM {table_name}
                WHERE {input_group_search_key} {where_condition}""".format(
            input_group_search_key=input_group_search_key,
            where_condition=where_condition,
            table_name=table_name,
        )
        result_db = sql(query)
        result_db_list = [obj.get(input_group_search_key) for obj in result_db]

        # if len(input_group_list) != len(result_db_list):

        def _match_value(search_in, search_for):
            _matched_count = 0
            for li in search_in:
                if input_group_operator == 'LIKE' and search_for in li:
                    _matched_count += 1
                elif input_group_operator == 'IN' and search_for == li:
                    _matched_count += 1
            return _matched_count

        for obj in input_group:
            if _match_value(result_db_list, obj.get("search_value")) == 0:
                error_obj = dict(
                    indexes=obj.get("indexes")
                )
                error_obj[input_group_search_key] = dict(
                    message="{} '{}' is not exists".format(input_group_search_key, obj.get("search_value"))
                )

                validation_error.append(error_obj)
            elif _match_value(result_db_list, obj.get("search_value")) > 1:
                error_obj = dict(
                    indexes=obj.get("indexes")
                )
                error_obj[input_group_search_key] = dict(
                    message="{} '{}' too many exists, can not determine witch one".format(input_group_search_key,
                                                                                          obj.get("search_value"))
                )

                validation_error.append(error_obj)
            else:
                matched_obj = _find_matched(db_results=result_db, key=input_group_search_key, search_item=obj,
                                            operator=input_group_operator)
                obj.update(dict(
                    matched_id=matched_obj.get("id")
                ))
                if append_extra and isinstance(append_extra, list):
                    for _append_key in append_extra:
                        obj[_append_key] = matched_obj.get(_append_key)

    return search_list, validation_error



import boto3
from app import app


def get_s3_client():
    region_name = app.config.get("S3_REGION")
    identity_path = app.config.get("AWS_WEB_IDENTITY_TOKEN_FILE")
    if identity_path:
        sts_file = open(identity_path, "r")
        sts_token = sts_file.read()

        role_arn = app.config.get("AWS_ROLE_ARN")

        sts_client = boto3.client("sts")
        response = sts_client.assume_role_with_web_identity(
            RoleArn=role_arn,
            RoleSessionName="sdd-s3",
            WebIdentityToken=sts_token,
            DurationSeconds=3600,
        )
        credential_dict = response["Credentials"]
        app.logger.info(credential_dict)
        s3_client = boto3.client(
            "s3",
            aws_access_key_id=credential_dict["AccessKeyId"],
            aws_secret_access_key=credential_dict["SecretAccessKey"],
            aws_session_token=credential_dict["SessionToken"],
            region_name=region_name,
        )
    elif app.config.get("AWS_SESSION_TOKEN"):
        s3_client = boto3.client(
            "s3",
            aws_access_key_id=app.config.get("AWS_ACCESS_KEY_ID"),
            aws_secret_access_key=app.config.get("AWS_SECRET_ACCESS_KEY"),
            aws_session_token=app.config.get("AWS_SESSION_TOKEN"),
            region_name=region_name,
        )
    else:
        s3_client = boto3.client(
            "s3",
            aws_access_key_id=app.config.get("AWS_ACCESS_KEY_ID"),
            aws_secret_access_key=app.config.get("AWS_SECRET_ACCESS_KEY"),
            region_name=region_name,
        )

    return s3_client



from wedeliver_core.wedeliver_core.helpers.amazon.get_file_url import get_file_url

from wedeliver_core.wedeliver_core.helpers.amazon.get_s3_client import get_s3_client


def append_plain_urls_to_list_dict(list_dict, key_name):
    """
    This function will append plain url to list of dictionaries
    :param list_dict: list of dictionaries
    :param key_name: key name to append plain url to
    :return: list of dictionaries
    """
    s3_client = get_s3_client()
    for item in list_dict:
        item[key_name] = get_file_url(item.get(key_name), s3_client=s3_client)
    return list_dict
